@@ -2,7 +2,7 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2002-2004
+ * Copyright (C) Richard P. Curnow  2002,2003,2004,2005
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -311,7 +311,7 @@ static void import_toktable(char *data, unsigned int hash_key, int n_msgs, struc
         /* No point going on - need to find out why the database got corrupted
          * in the 1st place.  Workaround for user - rebuild database from
          * scratch by deleting it then rerunning. */
-        exit(1);
+        unlock_and_exit(1);
       }
       ++index;
       index &= out->mask;
@@ -402,7 +402,7 @@ static void import_toktable2(char *data, unsigned int hash_key, int n_msgs, stru
   }
 }
 /*}}}*/
-struct database *new_database_from_file(char *db_filename)/*{{{*/
+struct database *new_database_from_file(char *db_filename, int do_integrity_checks)/*{{{*/
 { 
   /* Read existing database from file for doing incremental update */
   struct database *result;
@@ -492,7 +492,9 @@ struct database *new_database_from_file(char *db_filename)/*{{{*/
 
   close_db(input);
 
-  check_database_integrity(result);
+  if (do_integrity_checks) {
+    check_database_integrity(result);
+  }
   
   return result;
 }
@@ -683,7 +685,7 @@ static void scan_new_messages(struct database *db, int start_at)/*{{{*/
       free_rfc822(msg);
     }
     else
-      fprintf(stderr, "Skipping...\n");
+      fprintf(stderr, "Skipping %s (could not parse message)\n", db->msgs[i].src.mpf.path);
   }
 }
 /*}}}*/
@@ -1130,7 +1132,7 @@ static void recode_toktable2(struct toktable2 *tbl, int *new_idx)/*{{{*/
   }
 }
 /*}}}*/
-int cull_dead_messages(struct database *db)/*{{{*/
+int cull_dead_messages(struct database *db, int do_integrity_checks)/*{{{*/
 {
   /* Return true if any culled */
 
@@ -1139,7 +1141,9 @@ int cull_dead_messages(struct database *db)/*{{{*/
 
   /* Check db is OK before we start on this. (Check afterwards is done in the
    * writer.c code.) */
-  check_database_integrity(db);
+  if (do_integrity_checks) {
+    check_database_integrity(db);
+  }
 
   if (verbose) {
     fprintf(stderr, "Culling dead messages\n");
