@@ -2,7 +2,7 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2002-2004
+ * Copyright (C) Richard P. Curnow  2002,2003,2004,2005
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -183,7 +183,7 @@ static int has_child_dir(const char *base, const char *child)/*{{{*/
   return result;
 }
 /*}}}*/
-static int filter_is_maildir(const char *path, struct stat *sb)/*{{{*/
+int filter_is_maildir(const char *path, struct stat *sb)/*{{{*/
 {
   if (S_ISDIR(sb->st_mode)) {
     if (has_child_dir(path, "new") &&
@@ -195,7 +195,7 @@ static int filter_is_maildir(const char *path, struct stat *sb)/*{{{*/
   return 0;
 }
 /*}}}*/
-static int filter_is_mh(const char *path, struct stat *sb)/*{{{*/
+int filter_is_mh(const char *path, struct stat *sb)/*{{{*/
 {
   /* At this stage, just check it's a directory. */
   if (S_ISDIR(sb->st_mode)) {
@@ -283,98 +283,10 @@ static void sort_message_list(struct msgpath_array *arr)/*{{{*/
   qsort(arr->paths, arr->n, sizeof(struct msgpath), message_compare);
 }
 /*}}}*/
-static char *copy_folder_name(const char *start, const char *end)/*{{{*/
-{
-  /* 'start' points to start of string to copy.
-     Any '\:' sequence is replaced by ':' .
-     Otherwise \ is treated normally.
-     'end' can be 1 beyond the end of the string to copy.  Otherwise it can be
-     null, meaning treat 'start' as the start of a normal null-terminated
-     string. */
-  char *p;
-  const char *q;
-  int len;
-  char *result;
-  if (end) {
-    len = end - start;
-  } else {
-    len = strlen(start);
-  }
-  result = new_array(char, len + 1);
-  for (p=result, q=start;
-       end ? (q < end) : *q;
-       q++) {
-    if ((q[0] == '\\') && (q[1] == ':')) {
-      /* Escaped colon : drop the backslash */
-    } else {
-      *p++ = *q;
-    }
-  }
-  *p = '\0';
-  return result;
-}
-/*}}}*/
-void string_list_to_array(struct string_list *list, int *n, char ***arr)/*{{{*/
-{
-  int N, i;
-  struct string_list *a, *next_a;
-  char **result;
-  for (N=0, a=list->next; a!=list; a=a->next, N++) ;
-
-  result = new_array(char *, N);
-  for (i=0, a=list->next; i<N; a=next_a, i++) {
-    result[i] = a->data;
-    next_a = a->next;
-    free(a);
-  }
-
-  *n = N;
-  *arr = result;
-}
-/*}}}*/
-void split_on_colons(const char *str, int *n, char ***arr)/*{{{*/
-{
-  struct string_list list, *new_cell;
-  const char *left_to_do;
-
-  list.next = list.prev = &list;
-  left_to_do = str;
-  do {
-    char *colon;
-    char *xx;
-
-    colon = strchr(left_to_do, ':');
-    /* Allow backslash-escaped colons in filenames */
-    if (colon && (colon > left_to_do) && (colon[-1]=='\\')) {
-      int is_escaped;
-      do {
-        colon = strchr(colon + 1, ':');
-        is_escaped = (colon && (colon[-1] == '\\'));
-      } while (colon && is_escaped);
-    }
-    /* 'colon' now points to the first non-escaped colon or is null if there
-       were no more such colons in the rest of the line. */
-
-    xx = copy_folder_name(left_to_do, colon);
-    if (colon) {
-      left_to_do = colon + 1;
-    } else {
-      while (*left_to_do) ++left_to_do;
-    }
-
-    new_cell = new(struct string_list);
-    new_cell->data = xx;
-    new_cell->next = &list;
-    new_cell->prev = list.prev;
-    list.prev->next = new_cell;
-    list.prev = new_cell;
-  } while (*left_to_do);
-
-  string_list_to_array(&list, n, arr);
-
-}
-/*}}}*/
-void build_message_list(char *folder_base, char *folders, enum folder_type ft, struct msgpath_array *msgs)/*{{{*/
+/*{{{ void build_message_list */
+void build_message_list(char *folder_base, char *folders, enum folder_type ft,
+    struct msgpath_array *msgs,
+    struct globber_array *omit_globs)
 {
   char **raw_paths, **paths;
   int n_raw_paths, n_paths, i;
@@ -382,13 +294,13 @@ void build_message_list(char *folder_base, char *folders, enum folder_type ft, s
   split_on_colons(folders, &n_raw_paths, &raw_paths);
   switch (ft) {
     case FT_MAILDIR:
-      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_maildir);
+      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_maildir, omit_globs);
       for (i=0; i<n_paths; i++) {
         get_maildir_message_paths(paths[i], msgs);
       }
       break;
     case FT_MH:
-      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_mh);
+      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_mh, omit_globs);
       for (i=0; i<n_paths; i++) {
         get_mh_message_paths(paths[i], msgs);
       }
