@@ -2,7 +2,9 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2002-2004
+ * Copyright (C) Richard P. Curnow  2002,2003,2004,2005
+ * Copyright (C) Sanjoy Mahajan 2005
+ * - mfolder validation code
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -41,8 +43,10 @@ static char *maildir_folders = NULL;
 static char *mh_folders = NULL;
 static char *mboxen = NULL;
 static char *mfolder = NULL;
+static char *omit = NULL;
 static char *database_path = NULL;
 static enum folder_type output_folder_type = FT_MAILDIR;
+static int skip_integrity_checks = 0;
 
 static int file_exists(char *name)/*{{{*/
 {
@@ -54,13 +58,65 @@ static int file_exists(char *name)/*{{{*/
   return 1;
 }
 /*}}}*/
+/*{{{ member of*/
+/* returns 1 iff COMPLETE_MFOLDER (i.e. the match folder with
+   folder_base prepended if needed) matches one of the FOLDERS after
+   expanding the wildcards and recursion. Used to make sure that the
+   match folder will not overwrite a valuable mail file or
+   directory.  */
+int member_of (const char *complete_mfolder,
+    const char *folder_base,
+    const char *folders,
+    enum folder_type ft,
+    struct globber_array *omit_globs) {
+  char **raw_paths, **paths;
+  int n_raw_paths, n_paths, i;
+
+  if (!folders)
+    return 0;
+  split_on_colons(folders, &n_raw_paths, &raw_paths);
+  switch (ft) {
+    case FT_MAILDIR:
+      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_maildir, omit_globs);
+      break;
+    case FT_MH:
+      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_mh, omit_globs);
+      break;
+    case FT_MBOX:
+      glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_file, omit_globs);
+      break;
+    case FT_RAW:			/* cannot happen but to keep compiler happy */
+      break;    
+  }
+  for (i=0; i<n_paths; i++) {
+    struct stat mfolder_sb, src_folder_sb; /* for checking inode numbers */
+
+    /* if the complete path names are the same, definitely a match */
+    if (strcmp (complete_mfolder, paths[i]) == 0)
+      return 1;
+    /* also a match if they point to the same file or directory but
+       via different routes (e.g. absolute path for one but path with
+       ../.. for the other), so check inode numbers */
+    /* if cannot even get stat() info, probably not wrecking any mail
+       files or dirs, so continue, i.e. skip inode check. */
+    if (stat (complete_mfolder, &mfolder_sb) != 0 ||
+        stat (paths[i], &src_folder_sb) != 0)
+      continue;
+    if (mfolder_sb.st_ino == src_folder_sb.st_ino)
+      return 1;
+  }
+  return 0;  
+}
+/*}}}*/
 static char *copy_value(char *text)/*{{{*/
 {
   char *p;
+  char *result;
   for (p = text; *p && (*p != '='); p++) ;
   if (!*p) return NULL;
   p++;
-  return new_string(p);
+  result = expand_string(p);
+  return result;
 }
 /*}}}*/
 static void add_folders(char **folders, char *extra_folders)/*{{{*/
@@ -175,12 +231,14 @@ static void parse_rc_file(char *name)/*{{{*/
     }
     else if (!strncasecmp(p, "mh=", 3)) add_folders(&mh_folders, copy_value(p));
     else if (!strncasecmp(p, "mbox=", 5)) add_folders(&mboxen, copy_value(p));
+    else if (!strncasecmp(p, "omit=", 5)) add_folders(&omit, copy_value(p));
       
     else if (!strncasecmp(p, "mformat=", 8)) {
       parse_output_folder(p);
     }
     else if (!strncasecmp(p, "mfolder=", 8)) mfolder = copy_value(p);
     else if (!strncasecmp(p, "database=", 9)) database_path = copy_value(p);
+    else if (!strncasecmp(p, "nochecks", 8)) skip_integrity_checks = 1;
     else {
       if (verbose) {
         fprintf(stderr, "Unrecognized option at line %d in %s\n", lineno, name);
@@ -288,7 +346,7 @@ volatile void out_of_mem(char *file, int line, size_t size)/*{{{*/
 static void print_copyright(void)/*{{{*/
 {
   fprintf(stderr,
-          "mairix %s, Copyright (C) 2002-2004 Richard P. Curnow\n"
+          "mairix %s, Copyright (C) 2002-2005 Richard P. Curnow\n"
           "mairix comes with ABSOLUTELY NO WARRANTY.\n"
           "This is free software, and you are welcome to redistribute it\n"
           "under certain conditions; see the GNU General Public License for details.\n\n",
@@ -302,14 +360,16 @@ static void usage(void)/*{{{*/
   printf("mairix [-h]                                    : Show help\n"
          "mairix [-f <rcfile>] [-v] [-p]                 : Build index\n"
          "mairix [-f <rcfile>] [-a] [-t] expr1 ... exprN : Run search\n"
-         "-h          : show this help\n"
-         "-f <rcfile> : use alternative rc file (default ~/.mairixrc)\n"
-         "-v          : be verbose\n"
-         "-p          : purge messages that no longer exist\n"
-         "-a          : add new matches to match folder (default : clear it first)\n"
-         "-t          : include all messages in same threads as matching messages\n"
-         "-r          : force raw output regardless of mformat setting in mairixrc file\n"
-         "expr_i      : search expression (all expr's AND'ed together):\n"
+         "mairix [-f <rcfile>] -d                        : Dump database to stdout\n"
+         "-h           : show this help\n"
+         "-f <rcfile>  : use alternative rc file (default ~/.mairixrc)\n"
+         "-v           : be verbose\n"
+         "-p           : purge messages that no longer exist\n"
+         "-a           : add new matches to match folder (default : clear it first)\n"
+         "-t           : include all messages in same threads as matching messages\n"
+         "-o <mfolder> : override setting of mfolder from mairixrc file\n"
+         "-r           : force raw output regardless of mformat setting in mairixrc file\n"
+         "expr_i       : search expression (all expr's AND'ed together):\n"
          "    word          : match word in whole message\n"
          "    t:word        : match word in To: header\n"
          "    c:word        : match word in Cc: header\n"
@@ -322,10 +382,11 @@ static void usage(void)/*{{{*/
          "    d:start-end   : match date range\n"
          "    z:low-high    : match messages in size range\n"
          "    bs:word       : match word in Subject: header or body (or any other group of prefixes)\n"
-         "    s:word1+word2 : match both words in Subject:\n"
-         "    s:word1,word2 : match either word or both words in Subject:\n"
+         "    s:word1,word2 : match both words in Subject:\n"
+         "    s:word1/word2 : match either word or both words in Subject:\n"
          "    s:~word       : match messages not containing word in Subject:\n"
          "    s:substring=  : match substring in any word in Subject:\n"
+         "    s:^substring= : match left-anchored substring in any word in Subject:\n"
          "    s:substring=2 : match substring with <=2 errors in any word in Subject:\n"
          "\n"
          "    (See documentation for more examples)\n"
@@ -341,20 +402,21 @@ static void usage(void)/*{{{*/
    ancestor, because it'll pick up its own mfolders.  So, use environment
    variables to tailor the folders.
  
-   MAIRIX_FOLDER_BASE is the common ancestor directory of the folders (aka
+   MAIRIX_FOLDER_BASE is the common parent directory of the folders (aka
    mutt's 'folder' variable)
 
-   MAIRIX_FOLDERS is a colon-separated list of folders underneath that, with
-   the feature that '...' after a component means any maildir underneath that,
-   e.g.
+   MAIRIX_MAILDIR_FOLDERS, MAIRIX_MH_FOLDERS, MAIRIX_MBOXEN are
+   colon-separated lists of folders to index, with '...' after a
+   component meaning any maildir underneath it.
 
-   MAIRIX_MFOLDER is the parent of the mfolders underneath the base
-   
+   MAIRIX_MFOLDER is the folder to put the match data.
+
+   For example, if
    MAIRIX_FOLDER_BASE = "/home/foobar/mail"
    MAIRIX_FOLDERS = "inbox:lists...:action:archive..."
    MAIRIX_MFOLDER = "mf"
 
-   so /home/foobar/mail/vf/search1/{new,cur,tmp} contain the output for search1 etc.
+   then /home/foobar/mail/mf/{new,cur,tmp} contain the output of the search.
    }}} */
 
 int main (int argc, char **argv)/*{{{*/
@@ -374,6 +436,12 @@ int main (int argc, char **argv)/*{{{*/
   int do_help = 0;
   int do_raw_output = 0;
   int do_dump = 0;
+  int do_integrity_checks = 1;
+  int do_forced_unlock = 0;
+
+  struct globber_array *omit_globs;
+
+  int result;
 
   setlocale(LC_CTYPE, "");
 
@@ -398,6 +466,10 @@ int main (int argc, char **argv)/*{{{*/
       do_dump = 1;
     } else if (!strcmp(*argv, "-r") || !strcmp(*argv, "--raw-output")) {
       do_raw_output = 1;
+    } else if (!strcmp(*argv, "-Q") || !strcmp(*argv, "--no-integrity-checks")) {
+      do_integrity_checks = 0;
+    } else if (!strcmp(*argv, "--unlock")) {
+      do_forced_unlock = 1;
     } else if (!strcmp(*argv, "-v") || !strcmp(*argv, "--verbose")) {
       verbose = 1;
     } else if (!strcmp(*argv, "-h") ||
@@ -458,6 +530,10 @@ int main (int argc, char **argv)/*{{{*/
     mfolder = arg_mfolder;
   }
   
+  if (skip_integrity_checks) {
+    do_integrity_checks = 0;
+  }
+  
   if (!folder_base) {
     fprintf(stderr, "No folder_base/MAIRIX_FOLDER_BASE set\n");
     exit(2);
@@ -472,61 +548,103 @@ int main (int argc, char **argv)/*{{{*/
     output_folder_type = FT_RAW;
   }
 
+  if (omit) {
+    omit_globs = colon_sep_string_to_globber_array(omit);
+  } else {
+    omit_globs = NULL;
+  }
+
+  /* Lock database.
+   * Prevent concurrent updates due to parallel indexing (e.g. due to stuck
+   * cron jobs).
+   * Prevent concurrent searching and indexing. */
+
+  lock_database(database_path, do_forced_unlock);
+    
   if (do_dump) {
     dump_database(database_path);
-    return 0;
+    result = 0;
 
   } else if (do_search) {
+    int len;
+    char *complete_mfolder;
+
     if (!mfolder) {
       if (output_folder_type != FT_RAW) {
         fprintf(stderr, "No mfolder/MAIRIX_MFOLDER set\n");
-        exit(2);
+        unlock_and_exit(2);
       }
       mfolder = new_string("");
     }
 
-    return search_top(do_threads, do_augment, database_path, folder_base, mfolder, argv, output_folder_type, verbose);
+    /* complete_mfolder is needed by search_top() and member_of() so
+       compute it once here rather than in search_top() as well */
+    if ((mfolder[0] == '/') || 
+        ((mfolder[0] == '.') && (mfolder[1] == '/'))) {
+      complete_mfolder = new_string(mfolder);
+    } else {
+      len = strlen(folder_base) + strlen(mfolder) + 2;
+      complete_mfolder = new_array(char, len);
+      strcpy(complete_mfolder, folder_base);
+      strcat(complete_mfolder, "/");
+      strcat(complete_mfolder, mfolder);
+    }
+    /* check whether mfolder output would destroy a mail folder or mbox */
+    if (output_folder_type != FT_RAW &&	/* raw output cannot damage a folder */
+        (member_of(complete_mfolder,folder_base, maildir_folders, FT_MAILDIR, omit_globs)||
+         member_of (complete_mfolder, folder_base, mh_folders, FT_MH, omit_globs) ||
+         member_of (complete_mfolder, folder_base, mboxen, FT_MBOX, omit_globs))) {
+      fprintf (stderr,
+          "You asked search results to go to the folder '%s'.\n"
+          "That folder appears to be one of the indexed mail folders!\n"
+          "For your own good, I refuse to output search results to an indexed mail folder.\n",
+          mfolder);
+      unlock_and_exit(3);
+    }
+
+    result = search_top(do_threads, do_augment, database_path, complete_mfolder, argv, output_folder_type, verbose);
     
   } else {
+
     if (!maildir_folders && !mh_folders && !mboxen) {
       fprintf(stderr, "No [mh_]folders/mboxen/MAIRIX_[MH_]FOLDERS set\n");
-      exit(2);
+      unlock_and_exit(2);
     }
-    
+
     if (verbose) printf("Finding all currently existing messages...\n");
     msgs = new_msgpath_array();
     if (maildir_folders) {
-      build_message_list(folder_base, maildir_folders, FT_MAILDIR, msgs);
+      build_message_list(folder_base, maildir_folders, FT_MAILDIR, msgs, omit_globs);
     }
     if (mh_folders) {
-      build_message_list(folder_base, mh_folders, FT_MH, msgs);
+      build_message_list(folder_base, mh_folders, FT_MH, msgs, omit_globs);
     }
 
     /* The next call sorts the msgs array as part of looking for duplicates. */
     if (check_message_list_for_duplicates(msgs)) {
       fprintf(stderr, "Message list contains duplicates - check your 'folders' setting\n");
-      exit(2);
+      unlock_and_exit(2);
     }
 
     /* Try to open existing database */
     if (file_exists(database_path)) {
       if (verbose) printf("Reading existing database...\n");
-      db = new_database_from_file(database_path);
+      db = new_database_from_file(database_path, do_integrity_checks);
       if (verbose) printf("Loaded %d existing messages\n", db->n_msgs);
     } else {
       if (verbose) printf("Starting new database\n");
       db = new_database();
     }
 
-    build_mbox_lists(db, folder_base, mboxen);
+    build_mbox_lists(db, folder_base, mboxen, omit_globs);
     
     any_updates = update_database(db, msgs->paths, msgs->n);
     if (do_purge) {
-      any_purges = cull_dead_messages(db);
+      any_purges = cull_dead_messages(db, do_integrity_checks);
     }
-    if (1 || any_updates || any_purges) {
+    if (any_updates || any_purges) {
       /* For now write it every time.  This is obviously the most reliable method. */
-      write_database(db, database_path);
+      write_database(db, database_path, do_integrity_checks);
     }
 
 #if 0
@@ -536,8 +654,11 @@ int main (int argc, char **argv)/*{{{*/
     free_database(db);
     free_msgpath_array(msgs);
 
-    return 0;
+    result = 0;
   }
-  
+
+  unlock_database(); 
+
+  return result;
 }
 /*}}}*/
