@@ -2,7 +2,7 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2002-2004
+ * Copyright (C) Richard P. Curnow  2002,2003,2004,2005
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -233,6 +233,13 @@ struct string_list {/*{{{*/
 };
 /*}}}*/
 
+struct msg_src {
+  enum {MS_FILE, MS_MBOX} type;
+  char *filename;
+  off_t start;
+  size_t len;
+};
+
 extern int verbose; /* cmd line -v switch */
 
 /* Lame fix for systems where NAME_MAX isn't defined after including the above
@@ -243,6 +250,17 @@ extern int verbose; /* cmd line -v switch */
 #define NAME_MAX 4096
 #endif
 
+/* In glob.c */
+struct globber;
+struct globber_array;
+
+struct globber *make_globber(const char *wildstring);
+void free_globber(struct globber *old);
+int is_glob_match(struct globber *g, const char *s);
+struct globber_array *colon_sep_string_to_globber_array(const char *in);
+int is_globber_array_match(struct globber_array *ga, const char *s);
+void free_globber_array(struct globber_array *in);
+
 /* In hash.c */
 unsigned int hashfn( unsigned char *k, unsigned int length, unsigned int initval);
 
@@ -252,12 +270,15 @@ int is_integer_string(char *x);
 void free_msgpath_array(struct msgpath_array *x);
 void string_list_to_array(struct string_list *list, int *n, char ***arr);
 void split_on_colons(const char *str, int *n, char ***arr);
-void build_message_list(char *folder_base, char *folders, enum folder_type ft, struct msgpath_array *msgs);
+void build_message_list(char *folder_base, char *folders, enum folder_type ft,
+    struct msgpath_array *msgs, struct globber_array *omit_globs);
+int filter_is_maildir(const char *path, struct stat *sb);
+int filter_is_mh(const char *path, struct stat *sb);
   
 /* In rfc822.c */
 struct rfc822 *make_rfc822(char *filename);
 void free_rfc822(struct rfc822 *msg);
-struct rfc822 *data_to_rfc822(char *data, int length);
+struct rfc822 *data_to_rfc822(struct msg_src *src, char *data, int length);
 void create_ro_mapping(const char *filename, unsigned char **data, int *len);
 
 /* In tok.c */
@@ -274,23 +295,25 @@ void add_token2_in_file(int file_index, unsigned int hash_key, char *tok_text, s
 
 /* In db.c */
 struct database *new_database(void);
-struct database *new_database_from_file(char *db_filename);
+struct database *new_database_from_file(char *db_filename, int do_integrity_checks);
 void free_database(struct database *db);
 void maybe_grow_message_arrays(struct database *db);
 void tokenise_message(int file_index, struct database *db, struct rfc822 *msg);
 int update_database(struct database *db, struct msgpath *sorted_paths, int n_paths);
 void check_database_integrity(struct database *db);
-int cull_dead_messages(struct database *db);
+int cull_dead_messages(struct database *db, int do_integrity_checks);
 
 /* In mbox.c */
-void build_mbox_lists(struct database *db, const char *folder_base, const char *mboxen_paths);
+void build_mbox_lists(struct database *db, const char *folder_base,
+    const char *mboxen_paths, struct globber_array *omit_globs);
 int add_mbox_messages(struct database *db);
 void compute_checksum(const unsigned char *data, size_t len, checksum_t *csum);
 void cull_dead_mboxen(struct database *db);
 unsigned int encode_mbox_indices(unsigned int mb, unsigned int msg);
 void decode_mbox_indices(unsigned int index, unsigned int *mb, unsigned int *msg);
 int verify_mbox_size_constraints(struct database *db);
-void glob_and_expand_paths(const char *folder_base, char **paths_in, int n_in, char ***paths_out, int *n_out, int (*filter)(const char *, struct stat *));
+void glob_and_expand_paths(const char *folder_base, char **paths_in, int n_in, char ***paths_out, int *n_out, int (*filter)(const char *, struct stat *), struct globber_array *omit_globs);
+int filter_is_file(const char *x, struct stat *sb);
 
 /* In glob.c */
 struct globber;
@@ -300,10 +323,10 @@ void free_globber(struct globber *old);
 int is_glob_match(struct globber *g, const char *s);
 
 /* In writer.c */
-void write_database(struct database *db, char *filename);
+void write_database(struct database *db, char *filename, int do_integrity_checks);
 
 /* In search.c */
-int search_top(int do_threads, int do_augment, char *database_path, char *folder_base, char *mfolder, char **argv, enum folder_type ft, int verbose);
+int search_top(int do_threads, int do_augment, char *database_path, char *complete_mfolder, char **argv, enum folder_type ft, int verbose);
   
 /* In stats.c */
 void get_db_stats(struct database *db);
@@ -314,4 +337,12 @@ int scan_date_string(char *in, time_t *start, int *has_start, time_t *end, int *
 /* In dumper.c */
 void dump_database(char *filename);
 
+/* In strexpand.c */
+char *expand_string(const char *p);
+
+/* In dotlock.c */
+void lock_database(char *path, int forced_unlock);
+void unlock_database(void);
+void unlock_and_exit(int code);
+
 #endif /* MAIRIX_H */
