@@ -2,7 +2,7 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2003-2004
+ * Copyright (C) Richard P. Curnow  2003,2004,2005
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -228,7 +228,7 @@ static int looks_like_from_separator(off_t n, unsigned char *va, size_t len, int
 
   /* If we hit the end of the file, it doesn't look like a real 'From' line. */
 #ifdef DEBUG_DFA
-  exit(0);
+  unlock_and_exit(0);
 #endif
   return result;
 }
@@ -506,7 +506,7 @@ static void check_duplicates(struct extant_mbox *extant_mboxen, int n_extant)/*{
   }
   if (any_dupl) {
     printf("Exiting, the mairixrc file needs fixing\n");
-    exit(1);
+    unlock_and_exit(1);
   }
 }
 /*}}}*/
@@ -521,19 +521,25 @@ static char *find_last_slash(char *in)/*{{{*/
   return result;
 }
 /*}}}*/
-static void append_shallow(char *path, struct stat *sb, struct string_list *list, int (*filter)(const char *, struct stat *))/*{{{*/
+static void append_shallow(char *path, int base_len, struct stat *sb, struct string_list *list, /*{{{*/
+                  int (*filter)(const char *, struct stat *),
+                  struct globber_array *omit_globs)
 {
   if ((*filter)(path, sb)) {
-    struct string_list *nn = new(struct string_list);
-    nn->data = new_string(path);
-    nn->next = list;
-    nn->prev = list->prev;
-    list->prev->next = nn;
-    list->prev = nn;
+    if (!is_globber_array_match(omit_globs, path + base_len)) {
+      struct string_list *nn = new(struct string_list);
+      nn->data = new_string(path);
+      nn->next = list;
+      nn->prev = list->prev;
+      list->prev->next = nn;
+      list->prev = nn;
+    }
   }
 }
 /*}}}*/
-static void append_deep(char *path, struct stat *sb, struct string_list *list, int (*filter)(const char *, struct stat *))/*{{{*/
+static void append_deep(char *path, int base_len, struct stat *sb, struct string_list *list, /*{{{*/
+                        int (*filter)(const char *, struct stat *), 
+                        struct globber_array *omit_globs)
 {
   /* path is dir : read its contents, call append_shallow or self accordingly. */
   /* path is file : call append_shallow. */
@@ -542,7 +548,7 @@ static void append_deep(char *path, struct stat *sb, struct string_list *list, i
   DIR *d;
   struct dirent *de;
 
-  append_shallow(path, sb, list, filter);
+  append_shallow(path, base_len, sb, list, filter, omit_globs);
   
   if (S_ISDIR(sb->st_mode)) {
     xpath = new_array(char, strlen(path) + 2 + NAME_MAX);
@@ -554,11 +560,16 @@ static void append_deep(char *path, struct stat *sb, struct string_list *list, i
         strcpy(xpath, path);
         strcat(xpath, "/");
         strcat(xpath, de->d_name);
-        if (stat(xpath, &sb2) >= 0) {
-          if (S_ISREG(sb2.st_mode)) {
-            append_shallow(xpath, &sb2, list, filter);
-          } else if (S_ISDIR(sb2.st_mode)) {
-            append_deep(xpath, &sb2, list, filter);
+        if (!is_globber_array_match(omit_globs, xpath+base_len)) {
+          /* Filter out omissions at this point, e.g. to avoid wasting time on
+           * a recursive expansion of a tree that's going to get pruned in at
+           * the deepest level anyway. */
+          if (stat(xpath, &sb2) >= 0) {
+            if (S_ISREG(sb2.st_mode)) {
+              append_shallow(xpath, base_len, &sb2, list, filter, omit_globs);
+            } else if (S_ISDIR(sb2.st_mode)) {
+              append_deep(xpath, base_len, &sb2, list, filter, omit_globs);
+            }
           }
         }
       }
@@ -568,15 +579,18 @@ static void append_deep(char *path, struct stat *sb, struct string_list *list, i
   }
 }
 /*}}}*/
-static void handle_wild(char *path, char *last_comp, struct string_list *list,/*{{{*/
-                        void (*append)(char *, struct stat *, struct string_list *, int(*)(const char*, struct stat *)),
-                        int (*filter)(const char *, struct stat *))
+static void handle_wild(char *path, int base_len, char *last_comp, struct string_list *list,/*{{{*/
+                        void (*append)(char *, int, struct stat *, struct string_list *,
+                              int(*)(const char*, struct stat *), struct globber_array *),
+                        int (*filter)(const char *, struct stat *),
+                        struct globber_array *omit_globs)
 {
   /* last_comp is the character within 'path' where the wildcard stuff starts. */
   struct globber *gg;
   char *temp_path, *xpath;
   DIR *d;
   struct dirent *de;
+  int had_matches;
 
   gg = make_globber(last_comp);
 
@@ -593,6 +607,7 @@ static void handle_wild(char *path, char *last_comp, struct string_list *list,/*
   }
 
   d = opendir(temp_path);
+  had_matches = 0;
   if (d) {
     while ((de = readdir(d))) {
       if (!strcmp(de->d_name, ".")) continue;
@@ -602,30 +617,46 @@ static void handle_wild(char *path, char *last_comp, struct string_list *list,/*
         strcpy(xpath, temp_path);
         strcat(xpath, "/");
         strcat(xpath, de->d_name);
-        if (stat(xpath, &xsb) >= 0) {
-          (*append)(xpath, &xsb, list, filter);
+        if (!is_globber_array_match(omit_globs, xpath+base_len)) {
+          /* Filter out omissions at this point, e.g. to avoid wasting time on
+           * a recursive expansion of a tree that's going to get pruned in full
+           * later anyway. */
+          had_matches = 1;
+          if (stat(xpath, &xsb) >= 0) {
+            (*append)(xpath, base_len, &xsb, list, filter, omit_globs);
+          }
         }
       }
     }
     closedir(d);
+    if (!had_matches) {
+      fprintf(stderr, "WARNING: Wildcard \"%s\" matched nothing in %s\n", last_comp, temp_path);
+    }
+  } else {
+    fprintf(stderr, "WARNING: Folder path %s does not exist\n", temp_path);
   }
+
   
   free(temp_path);
   free(xpath);
   free(gg);
 }
 /*}}}*/
-static void handle_single(char *path, struct string_list *list,/*{{{*/
-                          void (*append)(char *, struct stat *, struct string_list *, int(*)(const char*, struct stat *)),
-                          int (*filter)(const char *, struct stat *))
+static void handle_single(char *path, int base_len, struct string_list *list,/*{{{*/
+                          void (*append)(char *, int, struct stat *, struct string_list *,
+                                int(*)(const char*, struct stat *), struct globber_array *),
+                          int (*filter)(const char *, struct stat *),
+                          struct globber_array *omit_globs)
 {
   struct stat sb;
   if (stat(path, &sb) >= 0) {
-    (*append)(path, &sb, list, filter);
+    (*append)(path, base_len, &sb, list, filter, omit_globs);
+  } else {
+    fprintf(stderr, "WARNING: Folder path %s does not exist\n", path);
   }
 }
 /*}}}*/
-static int filter_is_file(const char *x, struct stat *sb)/*{{{*/
+int filter_is_file(const char *x, struct stat *sb)/*{{{*/
 {
   if (S_ISREG(sb->st_mode))
     return 1;
@@ -649,7 +680,7 @@ static int is_wild(const char *x)/*{{{*/
   return 0;
 }
 /*}}}*/
-static void handle_one_path(const char *folder_base, const char *path, struct string_list *list, int (*filter)(const char *, struct stat *))/*{{{*/
+static void handle_one_path(const char *folder_base, const char *path, struct string_list *list, int (*filter)(const char *, struct stat *), struct globber_array *omit_globs)/*{{{*/
 {
   /* Valid syntaxen ([.]=optional):
    * [xxx/]foo : single path
@@ -670,15 +701,18 @@ static void handle_one_path(const char *folder_base, const char *path, struct st
   int len;
   char *last_slash;
   char *last_comp;
+  int base_len;
 
   is_abs = (path[0] == '/') ? 1 : 0;
   if (is_abs) {
     full_path = new_string(path);
+    base_len = 0;
   } else {
     full_path = new_array(char, folder_base_len + strlen(path) + 2);
     strcpy(full_path, folder_base);
     strcat(full_path, "/");
     strcat(full_path, path);
+    base_len = strlen(folder_base) + 1;
   }
   len = strlen(full_path);
   last_slash = find_last_slash(full_path);
@@ -686,21 +720,21 @@ static void handle_one_path(const char *folder_base, const char *path, struct st
   if ((len >= 4) && !strcmp(full_path + (len - 3), "...")) {
     full_path[len - 3] = '\0';
     if (is_wild(last_comp)) {
-      handle_wild(full_path, last_comp, list, append_deep, filter);
+      handle_wild(full_path, base_len, last_comp, list, append_deep, filter, omit_globs);
     } else {
-      handle_single(full_path, list, append_deep, filter);
+      handle_single(full_path, base_len, list, append_deep, filter, omit_globs);
     }
   } else {
     if (is_wild(last_comp)) {
-      handle_wild(full_path, last_comp, list, append_shallow, filter);
+      handle_wild(full_path, base_len, last_comp, list, append_shallow, filter, omit_globs);
     } else {
-      handle_single(full_path, list, append_shallow, filter);
+      handle_single(full_path, base_len, list, append_shallow, filter, omit_globs);
     }
   }
   free(full_path);
 }
 /*}}}*/
-void glob_and_expand_paths(const char *folder_base, char **paths_in, int n_in, char ***paths_out, int *n_out, int (*filter)(const char *, struct stat *))/*{{{*/
+void glob_and_expand_paths(const char *folder_base, char **paths_in, int n_in, char ***paths_out, int *n_out, int (*filter)(const char *, struct stat *), struct globber_array *omit_globs)/*{{{*/
 {
   struct string_list list;
   int i;
@@ -710,14 +744,15 @@ void glob_and_expand_paths(const char *folder_base, char **paths_in, int n_in, c
 
   for (i=0; i<n_in; i++) {
     char *path = paths_in[i];
-    handle_one_path(folder_base, path, &list, filter); 
+    handle_one_path(folder_base, path, &list, filter, omit_globs); 
   }
 
   string_list_to_array(&list, n_out, paths_out);
 }
 /*}}}*/
 
-void build_mbox_lists(struct database *db, const char *folder_base, const char *mboxen_paths)/*{{{*/
+void build_mbox_lists(struct database *db, const char *folder_base, /*{{{*/
+    const char *mboxen_paths, struct globber_array *omit_globs)
 {
   char **raw_paths, **paths;
   int n_raw_paths, i;
@@ -731,7 +766,7 @@ void build_mbox_lists(struct database *db, const char *folder_base, const char *
 
   if (mboxen_paths) {
     split_on_colons(mboxen_paths, &n_raw_paths, &raw_paths);
-    glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_file);
+    glob_and_expand_paths(folder_base, raw_paths, n_raw_paths, &paths, &n_paths, filter_is_file, omit_globs);
     extant_mboxen = new_array(struct extant_mbox, n_paths);
   } else {
     n_paths = 0;
@@ -802,6 +837,16 @@ void build_mbox_lists(struct database *db, const char *folder_base, const char *
 }
 /*}}}*/
 
+static struct msg_src *setup_msg_src(char *filename, off_t start, size_t len)/*{{{*/
+{
+  static struct msg_src result;
+  result.type = MS_MBOX;
+  result.filename = filename;
+  result.start = start;
+  result.len = len;
+  return &result;
+}
+/*}}}*/
 int add_mbox_messages(struct database *db)/*{{{*/
 {
   int i, j;
@@ -817,6 +862,7 @@ int add_mbox_messages(struct database *db)/*{{{*/
       off_t start;
       size_t len;
       struct rfc822 *r8;
+      struct msg_src *msg_src;
       maybe_grow_message_arrays(db);
       n = db->n_msgs;
       db->type[n] = MTY_MBOX;
@@ -828,12 +874,13 @@ int add_mbox_messages(struct database *db)/*{{{*/
       }
       if (!va) {
         fprintf(stderr, "Couldn't create mapping of file %s\n", mb->path);
-        exit(1);
+        unlock_and_exit(1);
       }
 
       start = mb->start[j];
       len   = mb->len[j];
-      r8 = data_to_rfc822(va + start, len);
+      msg_src = setup_msg_src(mb->path, start, len);
+      r8 = data_to_rfc822(msg_src, va + start, len);
       if (r8) {
         if (verbose) {
           printf("Scanning %s[%d] at [%d,%d)\n", mb->path, j, (int)start, (int)(start + len));
