@@ -2,7 +2,7 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2002-2004
+ * Copyright (C) Richard P. Curnow  2002,2003,2004,2005
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -85,7 +85,7 @@ struct write_map {/*{{{*/
 };
 /*}}}*/
 
-static char *create_rw_mapping(char *filename, size_t len)/*{{{*/
+static void create_rw_mapping(char *filename, size_t len, int *out_fd, char **out_data)/*{{{*/
 {
   int fd;
   char *data;
@@ -94,29 +94,29 @@ static char *create_rw_mapping(char *filename, size_t len)/*{{{*/
   fd = open(filename, O_RDWR | O_CREAT, 0600);
   if (fd < 0) {
     perror("open");
-    exit(2);
+    unlock_and_exit(2);
   }
 
   if (fstat(fd, &sb) < 0) {
     perror("stat");
-    exit(2);
+    unlock_and_exit(2);
   }
 
   if (sb.st_size < len) {
     /* Extend */
     if (lseek(fd, len - 1, SEEK_SET) < 0) {
       perror("lseek");
-      exit(2);
+      unlock_and_exit(2);
     }
     if (write(fd, "\000", 1) < 0) {
       perror("write");
-      exit(2);
+      unlock_and_exit(2);
     }
   } else if (sb.st_size > len) {
     /* Truncate */
     if (ftruncate(fd, len) < 0) {
       perror("ftruncate");
-      exit(2);
+      unlock_and_exit(2);
     }
   } else {
     /* Exactly the right length already - nothing to do! */
@@ -124,16 +124,12 @@ static char *create_rw_mapping(char *filename, size_t len)/*{{{*/
 
   data = mmap(0, len, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
   if (data == MAP_FAILED) {
-    perror("mmap");
-    exit(2);
+    perror("writer:mmap");
+    unlock_and_exit(2);
   }
 
-  if (close(fd) < 0) {
-    perror("close");
-    exit(2);
-  }
-  
-  return data;
+  *out_data = data;
+  *out_fd = fd;
 }
 /*}}}*/
   
@@ -548,17 +544,20 @@ static char *write_toktable2(struct toktable2 *tab, struct write_map_toktable2 *
   return cdata;
 }
 /*}}}*/
-void write_database(struct database *db, char *filename)/*{{{*/
+void write_database(struct database *db, char *filename, int do_integrity_checks)/*{{{*/
 {
   int file_len;
+  int fd;
   char *data, *cdata;
   unsigned int *uidata;
   struct write_map map;
 
-  check_database_integrity(db);
+  if (do_integrity_checks) {
+    check_database_integrity(db);
+  }
 
   if (!verify_mbox_size_constraints(db)) {
-    exit(1);
+    unlock_and_exit(1);
   }
   
   /* Work out mappings */
@@ -566,7 +565,7 @@ void write_database(struct database *db, char *filename)/*{{{*/
   
   file_len = char_length(db) + (4 * map.beyond_last_ui_offset);
   
-  data = create_rw_mapping(filename, file_len);
+  create_rw_mapping(filename, file_len, &fd, &data);
   uidata = (unsigned int *) data; /* align(int) < align(page)! */
   cdata = data + (4 * map.beyond_last_ui_offset);
 
@@ -586,7 +585,15 @@ void write_database(struct database *db, char *filename)/*{{{*/
   /* Unmap / close file */
   if (munmap(data, file_len) < 0) {
     perror("munmap");
-    exit(2);
+    unlock_and_exit(2);
+  }
+  if (fsync(fd) < 0) {
+    perror("fsync");
+    unlock_and_exit(2);
+  }
+  if (close(fd) < 0) {
+    perror("close");
+    unlock_and_exit(2);
   }
 }
   /*}}}*/
